
# q-expansion of newform 60.12.h.b, downloaded from the LMFDB on 19 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 60
weight = 12
poly_data = [1, 1, 2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[1, -1, 1, 0], 1], [[1, 1, -1, 1], 1], [[1, 3, -1, 1], 1]]

hecke_ring_character_values = [[31, [-1, 0, 0, 0]], [41, [-1, 0, 0, 0]], [37, [-1, 0, 0, 0]]]
aps_data = [[0, -10, 33, 0], [0, -243, 243, 0], [0, -3125, -3125, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -115454, -115454, 0], [-2822822, 0, 0, 5645644], [0, 28422286, -28422286, 0], [0, 0, 0, 0], [-67565122, 0, 0, 135130244], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -66814842, 66814842, 0], [0, -785310154, -785310154, 0], [0, 0, 0, 0], [13027614598, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [13694120222, 0, 0, -27388240444], [0, 35683656314, -35683656314, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 79427571042, -79427571042, 0], [-312857272714, 0, 0, 0], [0, -126915984446, -126915984446, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -498517915270, -498517915270, 0], [-94651252734, 0, 0, 189302505468], [0, 0, 0, 0], [435837424566, 0, 0, -871674849132], [0, 0, 0, 0], [0, 0, 0, 0], [0, 1424115162302, -1424115162302, 0], [0, -1793335644770, -1793335644770, 0], [0, 0, 0, 0], [4670725692622, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 3658419999254, 3658419999254, 0], [-2135830607466, 0, 0, 4271661214932], [539355432922, 0, 0, -1078710865844], [0, 0, 0, 0], [0, 7963811647498, -7963811647498, 0], [-17754695206274, 0, 0, 0], [0, -3510288428230, -3510288428230, 0], [0, 0, 0, 0], [-16435647386398, 0, 0, 0], [0, 0, 0, 0], [0, 11808927229778, 11808927229778, 0], [0, -23507705009826, 23507705009826, 0], [0, 0, 0, 0], [-6272593543522, 0, 0, 12545187087044], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -4240925378522, -4240925378522, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 47227868374270, 47227868374270, 0], [-36310783491966, 0, 0, 72621566983932], [0, 0, 0, 0], [0, -3457334044158, 3457334044158, 0], [1831148704934, 0, 0, 0], [0, -90873097271278, -90873097271278, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-33394808756110, 0, 0, 66789617512220], [0, 153124596659254, -153124596659254, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-455913804355126, 0, 0, 0], [0, 0, 0, 0], [-238099244096962, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-69184924356890, 0, 0, 138369848713780], [0, 47847173388146, -47847173388146, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -147294489273302, 147294489273302, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [335735297965978, 0, 0, -671470595931956], [0, -626660049454514, 626660049454514, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [982248272339398, 0, 0, 0], [0, 0, 0, 0], [0, -853608486373730, -853608486373730, 0], [0, 1549028258786826, -1549028258786826, 0], [0, 0, 0, 0], [-617136026821390, 0, 0, 1234272053642780], [0, 0, 0, 0], [0, 1024290635812418, -1024290635812418, 0], [0, -1353783866309246, -1353783866309246, 0], [0, 0, 0, 0], [-1244122330030838, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -63015526316006, -63015526316006, 0], [941692861804866, 0, 0, -1883385723609732], [331199930188390, 0, 0, -662399860376780], [0, 0, 0, 0], [0, 0, 0, 0], [0, -2429818226612098, 2429818226612098, 0], [0, -2693744555698306, -2693744555698306, 0], [0, 0, 0, 0], [6414715970935822, 0, 0, 0], [0, 0, 0, 0], [0, -2413007319145178, -2413007319145178, 0], [0, -1905975294422254, 1905975294422254, 0], [2135191573469722, 0, 0, -4270383146939444], [0, 0, 0, 0], [-5943972665335874, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [2133316274798954, 0, 0, -4266632549597908], [0, -2864640688525026, 2864640688525026, 0], [137025265660654, 0, 0, -274050531321308], [0, 0, 0, 0], [0, 0, 0, 0], [-14497466821588654, 0, 0, 0], [0, 5314382124913046, 5314382124913046, 0], [0, 0, 0, 0], [0, 6513845573034206, 6513845573034206, 0], [0, 0, 0, 0], [5138727424733634, 0, 0, -10277454849467268], [0, 0, 0, 0], [0, 0, 0, 0], [0, -5807257662276382, 5807257662276382, 0], [13901570782370534, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 3182997131414698, 3182997131414698, 0], [7052668031187890, 0, 0, -14105336062375780], [0, 16170038186488054, -16170038186488054, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -1645007149207762, 1645007149207762, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-9507954743905866, 0, 0, 19015909487811732], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 26922448966991962, -26922448966991962, 0], [0, -17471164590818198, -17471164590818198, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -17636035608797054, -17636035608797054, 0], [0, -18980922898275314, 18980922898275314, 0], [-14197307576522546, 0, 0, 28394615153045092], [0, 0, 0, 0]]
